@@ -3,15 +3,14 @@
  * The Ingo_Script_imap:: class represents an IMAP client-side script
  * generator.
  *
- * $Horde: ingo/lib/Script/imap.php,v 1.49.10.9 2006/02/12 20:59:38 slusarz Exp $
+ * $Horde: ingo/lib/Script/imap.php,v 1.49.10.15 2008/01/02 11:32:11 jan Exp $
  *
- * Copyright 2003-2006 Michael Slusarz <slusarz@bigworm.curecanti.org>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/asl.php.
  *
- * @author  Michael Slusarz <slusarz@bigworm.curecanti.org>
- * @since   Ingo 1.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Ingo
  */
 class Ingo_Script_imap extends Ingo_Script {
@@ -80,6 +79,13 @@ class Ingo_Script_imap extends Ingo_Script {
      */
     var $_ondemand = true;
 
+    /**
+     * The API to use for IMAP functions.
+     *
+     * @var Ingo_Script_imap_api
+     */
+    var $_api;
+
     /**
      * Perform the filtering specified in the rules.
      *
@@ -95,24 +101,25 @@ class Ingo_Script_imap extends Ingo_Script {
     {
         global $ingo_storage, $notification, $prefs;
 
-        $whitelist_ids = array();
+        if (empty($params['api'])) {
+            $this->_api = Ingo_Script_imap_api::factory('live', $params);
+        } else {
+            $this->_api = &$params['api'];
+        }
 
-        /* Get the IMAP_Cache object. */
-        require_once 'Horde/IMAP/Cache.php';
-        $imap_cache = &IMAP_Cache::singleton();
+        /* Indices that will be ignored by subsequent rules. */
+        $ignore_ids = array();
 
         /* Only do filtering if:
            1. We have not done filtering before -or-
            2. The mailbox has changed -or-
            3. The rules have changed. */
-        $cache = $imap_cache->getCache($params['imap'], $params['mailbox'], 'ingochange');
+        $cache = $this->_api->getCache();
         if (($cache !== false) && ($cache == $_SESSION['ingo']['change'])) {
             return true;
         }
 
         require_once 'Horde/MIME.php';
-        require_once INGO_BASE . '/lib/IMAP/Search.php';
-        $imap_search = &Ingo_IMAP_Search::singleton(array('imap' => $params['imap']));
 
         /* Grab the rules list. */
         $filters = &$ingo_storage->retrieve(INGO_STORAGE_ACTION_FILTERS);
@@ -133,7 +140,6 @@ class Ingo_Script_imap extends Ingo_Script {
             }
 
             $search_array = array();
-            $stop_flag = false;
 
             switch ($rule['action']) {
             case INGO_STORAGE_ACTION_BLACKLIST:
@@ -154,9 +160,10 @@ class Ingo_Script_imap extends Ingo_Script {
                     continue;
                 }
 
-                $query = &new IMAP_Search_Query();
+                require_once INGO_BASE . '/lib/IMAP/Search.php';
+                $query = new Ingo_IMAP_Search_Query();
                 foreach ($addr as $val) {
-                    $ob = &new IMAP_Search_Query();
+                    $ob = new Ingo_IMAP_Search_Query();
                     $ob->deleted(false);
                     if ($seen_flag == INGO_SCRIPT_FILTER_UNSEEN) {
                         $ob->seen(false);
@@ -164,34 +171,52 @@ class Ingo_Script_imap extends Ingo_Script {
                         $ob->seen(true);
                     }
                     $ob->header('from', $val);
-                    $search_array[] = &$ob;
+                    $search_array[] = $ob;
                 }
                 $query->imapOr($search_array);
-                $indices = $imap_search->searchMailbox($query, $params['imap'], $params['mailbox']);
+                $indices = $this->_api->search($query);
+
+                /* Remove any indices that got in there by way of partial
+                 * address match. */
+                $sequence = implode(',', $indices);
+                $msgs = $this->_api->fetchMessageOverviews($sequence);
+                foreach ($msgs as $msg) {
+                    $from_addr = MIME::bareAddress($msg->from);
+                    $found = false;
+                    foreach ($addr as $val) {
+                        if (strtolower($from_addr) == strtolower($val)) {
+                            $found = true;
+                        }
+                    }
+                    if (!$found) {
+                        $indices = array_diff($indices, array($msg->uid));
+                    }
+                }
 
                 if ($rule['action'] == INGO_STORAGE_ACTION_BLACKLIST) {
-                    $indices = array_diff($indices, $whitelist_ids);
+                    $indices = array_diff($indices, $ignore_ids);
                     if (!empty($indices)) {
                         $sequence = implode(',', $indices);
                         if (!empty($bl_folder)) {
-                            @imap_mail_move($params['imap'], $sequence, $bl_folder, CP_UID);
+                            $this->_api->moveMessages($sequence, $bl_folder);
                         } else {
-                            @imap_delete($params['imap'], $sequence, FT_UID);
+                            $this->_api->deleteMessages($sequence);
                         }
-                        @imap_expunge($params['imap']);
+                        $this->_api->expunge($indices);
                         $notification->push(sprintf(_("Filter activity: %s message(s) that matched the blacklist were deleted."), count($indices)), 'horde.message');
                     }
                 } else {
-                    $whitelist_ids = $indices;
+                    $ignore_ids = $indices;
                 }
                 break;
 
             case INGO_STORAGE_ACTION_KEEP:
             case INGO_STORAGE_ACTION_MOVE:
             case INGO_STORAGE_ACTION_DISCARD:
-                $query = &new IMAP_Search_Query();
+                require_once INGO_BASE . '/lib/IMAP/Search.php';
+                $query = new Ingo_IMAP_Search_Query();
                 foreach ($rule['conditions'] as $val) {
-                    $ob = &new IMAP_Search_Query();
+                    $ob = new Ingo_IMAP_Search_Query();
                     $ob->deleted(false);
                     if ($seen_flag == INGO_SCRIPT_FILTER_UNSEEN) {
                         $ob->seen(false);
@@ -220,7 +245,7 @@ class Ingo_Script_imap extends Ingo_Script {
                             $ob->header($val['field'], $val['value'], true);
                         }
                     }
-                    $search_array[] = &$ob;
+                    $search_array[] = $ob;
                 }
 
                 if ($rule['combine'] == INGO_STORAGE_COMBINE_ALL) {
@@ -229,11 +254,17 @@ class Ingo_Script_imap extends Ingo_Script {
                     $query->imapOr($search_array);
                 }
 
-                $indices = $imap_search->searchMailbox($query, $params['imap'], $params['mailbox']);
+                $indices = $this->_api->search($query);
+
+                if (($indices = array_diff($indices, $ignore_ids))) {
+                    if ($rule['stop']) {
+                        /* If the stop action is set, add these
+                         * indices to the list of ids that will be
+                         * ignored by subsequent rules. */
+                        $ignore_ids = array_unique($indices + $ignore_ids);
+                    }
 
-                if (($indices = array_diff($indices, $whitelist_ids))) {
                     $sequence = implode(',', $indices);
-                    $stop_flag = true;
 
                     /* Set the flags. */
                     if (!empty($rule['flags']) &&
@@ -251,21 +282,23 @@ class Ingo_Script_imap extends Ingo_Script {
                         if ($rule['flags'] & INGO_STORAGE_FLAG_SEEN) {
                             $flags[] = '\\Seen';
                         }
-                        @imap_setflag_full($params['imap'], $sequence, implode(' ', $flags), ST_UID);
+                        $this->_api->setMessageFlags($sequence,
+                                                     implode(' ', $flags));
                     }
 
                     if ($rule['action'] == INGO_STORAGE_ACTION_KEEP) {
-                        /* Add these indices to the current whitelist. */
-                        $whitelist_ids = array_unique($indices + $whitelist_ids);
+                        /* Add these indices to the ignore list. */
+                        $ignore_ids = array_unique($indices + $ignore_ids);
                     } elseif ($rule['action'] == INGO_STORAGE_ACTION_MOVE) {
                         /* We need to grab the overview first. */
                         if ($detailmsg) {
-                            $overview = @imap_fetch_overview($params['imap'], $sequence, FT_UID);
+                            $overview = $this->_api->fetchMessageOverviews($sequence);
                         }
 
                         /* Move the messages to the requested mailbox. */
-                        @imap_mail_move($params['imap'], $sequence, $rule['action-value'], CP_UID);
-                        @imap_expunge($params['imap']);
+                        $this->_api->moveMessages($sequence,
+                                                  $rule['action-value']);
+                        $this->_api->expunge($indices);
 
                         /* Display notification message(s). */
                         if ($detailmsg) {
@@ -283,12 +316,12 @@ class Ingo_Script_imap extends Ingo_Script {
                     } elseif ($rule['action'] == INGO_STORAGE_ACTION_DISCARD) {
                         /* We need to grab the overview first. */
                         if ($detailmsg) {
-                            $overview = @imap_fetch_overview($params['imap'], $sequence, FT_UID);
+                            $overview = $this->_api->fetchMessageOverviews($sequence);
                         }
 
                         /* Delete the messages now. */
-                        @imap_delete($params['imap'], $sequence, FT_UID);
-                        @imap_expunge($params['imap']);
+                        $this->_api->deleteMessages($sequence);
+                        $this->_api->expunge($indices);
 
                         /* Display notification message(s). */
                         if ($detailmsg) {
@@ -302,11 +335,12 @@ class Ingo_Script_imap extends Ingo_Script {
                         }
                     } elseif ($rule['action'] == INGO_STORAGE_ACTION_MOVEKEEP) {
                         /* Copy the messages to the requested mailbox. */
-                        @imap_mail_copy($params['imap'], $sequence, $rule['action-value'], CP_UID);
+                        $this->_api->copyMessages($sequence,
+                                                 $rule['action-value']);
 
                         /* Display notification message(s). */
                         if ($detailmsg) {
-                            $overview = @imap_fetch_overview($params['imap'], $sequence, FT_UID);
+                            $overview = $this->_api->fetchMessageOverviews($sequence);
                             foreach ($overview as $msg) {
                                 $notification->push(sprintf(_("Filter activity: The message \"%s\" from \"%s\" has been copied to the folder \"%s\"."), isset($msg->subject) ? MIME::decode($msg->subject, NLS::getCharset()) : _("[No Subject]"), MIME::decode($msg->from, NLS::getCharset()), $rule['action-value']), 'horde.message');
                             }
@@ -317,16 +351,10 @@ class Ingo_Script_imap extends Ingo_Script {
                 }
                 break;
             }
-
-            /* Handle stop flag. */
-            if ($stop_flag && $rule['stop']) {
-                break;
-            }
         }
 
         /* Set cache flag. */
-        $imap_cache->storeCache($params['imap'], $params['mailbox'], array('ingochange' => $_SESSION['ingo']['change']));
-
+        $this->_api->storeCache($_SESSION['ingo']['change']);
         return true;
     }
 
@@ -364,3 +392,67 @@ class Ingo_Script_imap extends Ingo_Script {
     }
 
 }
+
+class Ingo_Script_imap_api {
+
+    var $_params;
+
+    function Ingo_Script_imap_api($params = array())
+    {
+        $this->_params = $params;
+    }
+
+    function factory($type, $params)
+    {
+        $class = 'Ingo_Script_imap_' . $type;
+        if (!class_exists($class)) {
+            require dirname(__FILE__) . '/imap/' . $type . '.php';
+        }
+        return new $class($params);
+    }
+
+    function deleteMessages($sequence)
+    {
+        return PEAR::raiseError('Not implemented.');
+    }
+
+    function expunge($indices)
+    {
+        return PEAR::raiseError('Not implemented.');
+    }
+
+    function moveMessages($sequence, $folder)
+    {
+        return PEAR::raiseError('Not implemented.');
+    }
+
+    function copyMessages($sequence, $folder)
+    {
+        return PEAR::raiseError('Not implemented.');
+    }
+
+    function setMessageFlags($sequence, $flags)
+    {
+        return PEAR::raiseError('Not implemented.');
+    }
+
+    function fetchMessageOverviews($sequence)
+    {
+        return PEAR::raiseError('Not implemented.');
+    }
+
+    function search($query)
+    {
+        return PEAR::raiseError('Not implemented.');
+    }
+
+    function getCache()
+    {
+        return false;
+    }
+
+    function storeCache($timestamp)
+    {
+    }
+
+}
