@@ -1,4 +1,12 @@
 <?php
+/**
+ * $Horde: ingo/lib/Storage.php,v 1.43.8.18 2008/02/28 18:40:28 jan Exp $
+ *
+ * See the enclosed file LICENSE for license information (ASL).  If you
+ * did not receive this file, see http://www.horde.org/licenses/asl.php.
+ *
+ * @package Ingo
+ */
 
 /**
  * Ingo_Storage:: 'combine' constants
@@ -23,6 +31,7 @@ define('INGO_STORAGE_ACTION_FORWARD', 10);
 define('INGO_STORAGE_ACTION_MOVEKEEP', 11);
 define('INGO_STORAGE_ACTION_FLAGONLY', 12);
 define('INGO_STORAGE_ACTION_NOTIFY', 13);
+define('INGO_STORAGE_ACTION_SPAM', 14);
 
 /**
  * Ingo_Storage:: 'flags' constants
@@ -42,13 +51,8 @@ define('INGO_STORAGE_TYPE_BODY', 3);
 /**
  * Ingo_Storage:: defines an API to store the various filter rules.
  *
- * $Horde: ingo/lib/Storage.php,v 1.43.8.16 2006/05/01 13:17:38 jan Exp $
- *
- * See the enclosed file LICENSE for license information (ASL).  If you
- * did not receive this file, see http://www.horde.org/licenses/asl.php.
- *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   Ingo 0.1
+ * @author  Michael Slusarz <slusarz@horde.org>
+ * @author  Jan Schneider <jan@horde.org>
  * @package Ingo
  */
 class Ingo_Storage {
@@ -86,7 +90,7 @@ class Ingo_Storage {
      * @return mixed  The newly created concrete Ingo_Storage instance, or
      *                false on an error.
      */
-    function &factory($driver = null, $params = null)
+    function factory($driver = null, $params = null)
     {
         if (is_null($driver)) {
             $driver = $GLOBALS['conf']['storage']['driver'];
@@ -101,68 +105,26 @@ class Ingo_Storage {
         require_once dirname(__FILE__) . '/Storage/' . $driver . '.php';
         $class = 'Ingo_Storage_' . $driver;
         if (class_exists($class)) {
-            $ingo = &new $class($params);
+            return new $class($params);
         } else {
-            $ingo = false;
+            return false;
         }
-
-        return $ingo;
-    }
-
-    /**
-     * Attempts to return a reference to a concrete Ingo_Storage instance
-     * based on $driver.  It will only create a new instance if no
-     * Ingo_Storage instance with the same parameters currently exists.
-     *
-     * This should be used if multiple storage sources are required.
-     *
-     * This method must be invoked as: $var = &Ingo_Storage::singleton();
-     *
-     * @param string $driver  The type of concrete Ingo_Storage subclass to
-     *                        return.  This is based on the storage driver
-     *                        ($driver).  The code is dynamically included.
-     * @param array $params  A hash containing any additional configuration or
-     *                       connection parameters a subclass might need.
-     *
-     * @return mixed  The created concrete Ingo_Storage instance, or false
-     *                on error.
-     */
-    function &singleton($driver = null, $params = null)
-    {
-        static $instances;
-
-        if (is_null($driver)) {
-            $driver = $GLOBALS['conf']['storage']['driver'];
-        }
-
-        if (is_null($params)) {
-            $params = Horde::getDriverConfig('storage', $driver);
-        }
-
-        if (!isset($instances)) {
-            $instances = array();
-        }
-
-        $signature = serialize(array($driver, $params));
-        if (!isset($instances[$signature])) {
-            $instances[$signature] = &Ingo_Storage::factory($driver, $params);
-        }
-
-        return $instances[$signature];
     }
 
     /**
      * Retrieves the specified data.
      *
-     * @param integer $field  The field name of the desired data
-     *                        (INGO_STORAGE_ACTION_* constants).
-     * @param boolean $cache  Use the cached object?
+     * @param integer $field     The field name of the desired data
+     *                           (INGO_STORAGE_ACTION_* constants).
+     * @param boolean $cache     Use the cached object?
+     * @param boolean $readonly  Whether to disable any write operations.
      *
      * @return Ingo_Storage_rule|Ingo_Storage_filters  The specified object.
      */
-    function &retrieve($field, $cache = true)
+    function &retrieve($field, $cache = true, $readonly = false)
     {
-        if ($cache) {
+        /* Don't cache if using shares. */
+        if ($cache && empty($GLOBALS['ingo_shares'])) {
             if (!isset($this->_cache[$field])) {
                 $this->_cache[$field] = array('mod' => false);
                 if (isset($_SESSION['ingo']['storage'][$field])) {
@@ -170,7 +132,7 @@ class Ingo_Storage {
                     $cacheSess = &Horde_SessionObjects::singleton();
                     $this->_cache[$field]['ob'] = $cacheSess->query($_SESSION['ingo']['storage'][$field]);
                 } else {
-                    $this->_cache[$field]['ob'] = &$this->_retrieve($field);
+                    $this->_cache[$field]['ob'] = &$this->_retrieve($field, $readonly);
                 }
                 if (!$this->_shutdownCache) {
                     register_shutdown_function(array(&$this, '_addCacheShutdown'));
@@ -179,7 +141,7 @@ class Ingo_Storage {
             }
             $ob = &$this->_cache[$field]['ob'];
         } else {
-            $ob = &$this->_retrieve($field);
+            $ob = &$this->_retrieve($field, $readonly);
         }
 
         return $ob;
@@ -189,14 +151,14 @@ class Ingo_Storage {
      * Retrieves the specified data from the storage backend.
      *
      * @abstract
-     * @access private
      *
-     * @param integer $field  The field name of the desired data.
-     *                        See lib/Storage.php for the available fields.
+     * @param integer $field     The field name of the desired data.
+     *                           See lib/Storage.php for the available fields.
+     * @param boolean $readonly  Whether to disable any write operations.
      *
      * @return Ingo_Storage_rule|Ingo_Storage_filters  The specified data.
      */
-    function _retrieve($field)
+    function _retrieve($field, $readonly = false)
     {
         return false;
     }
@@ -215,7 +177,8 @@ class Ingo_Storage {
         if (in_array($type, array(INGO_STORAGE_ACTION_BLACKLIST,
                                   INGO_STORAGE_ACTION_VACATION,
                                   INGO_STORAGE_ACTION_WHITELIST,
-                                  INGO_STORAGE_ACTION_FORWARD))) {
+                                  INGO_STORAGE_ACTION_FORWARD,
+                                  INGO_STORAGE_ACTION_SPAM))) {
             $filters = $this->retrieve(INGO_STORAGE_ACTION_FILTERS);
             if ($filters->findRuleId($type) === null) {
                 switch ($type) {
@@ -231,6 +194,9 @@ class Ingo_Storage {
                 case INGO_STORAGE_ACTION_FORWARD:
                     $name = 'Forward';
                     break;
+                case INGO_STORAGE_ACTION_SPAM:
+                    $name = 'Spam Filter';
+                    break;
                 }
                 $filters->addRule(array('action' => $type, 'name' => $name));
                 $result = $this->store($filters, $cache);
@@ -240,6 +206,7 @@ class Ingo_Storage {
             }
         }
 
+        $result = $this->_store($ob);
         if ($cache) {
             $this->_cache[$ob->obType()] = array('ob' => $ob, 'mod' => true);
             if (!$this->_shutdownCache) {
@@ -248,7 +215,7 @@ class Ingo_Storage {
             }
         }
 
-        return $this->_store($ob);
+        return $result;
     }
 
     /**
@@ -428,7 +395,7 @@ class Ingo_Storage {
  * Ingo_Storage_rule:: is the base class for the various action objects
  * used by Ingo_Storage.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Ingo 1.0
  * @package Ingo
  */
@@ -441,6 +408,13 @@ class Ingo_Storage_rule {
      */
     var $_obtype;
 
+    /**
+     * Whether the rule has been saved (if being saved separately).
+     *
+     * @var boolean
+     */
+    var $_saved = false;
+
     /**
      * Returns the object rule type.
      *
@@ -451,6 +425,26 @@ class Ingo_Storage_rule {
         return $this->_obtype;
     }
 
+    /**
+     * Marks the rule as saved or unsaved.
+     *
+     * @param boolean $data  Whether the rule has been saved.
+     */
+    function setSaved($data)
+    {
+        $this->_saved = $data;
+    }
+
+    /**
+     * Returns whether the rule has been saved.
+     *
+     * @return boolean  True if the rule has been saved.
+     */
+    function isSaved()
+    {
+        return $this->_saved;
+    }
+
     /**
      * Function to manage an internal address list.
      *
@@ -486,7 +480,7 @@ class Ingo_Storage_rule {
  * Ingo_Storage_blacklist is the object used to hold blacklist rule
  * information.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Ingo 1.0
  * @package Ingo
  */
@@ -539,7 +533,7 @@ class Ingo_Storage_blacklist extends Ingo_Storage_rule {
  * Ingo_Storage_whitelist is the object used to hold whitelist rule
  * information.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Ingo 1.0
  * @package Ingo
  */
@@ -582,7 +576,7 @@ class Ingo_Storage_whitelist extends Ingo_Storage_rule {
  * Ingo_Storage_forward is the object used to hold mail forwarding rule
  * information.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Ingo 1.0
  * @package Ingo
  */
@@ -604,6 +598,13 @@ class Ingo_Storage_forward extends Ingo_Storage_rule {
 
     function getForwardAddresses()
     {
+        if (is_array($this->_addr)) {
+            foreach ($this->_addr as $key => $val) {
+                if (empty($val)) {
+                    unset($this->_addr[$key]);
+                }
+            }
+        }
         return $this->_addr;
     }
 
@@ -618,7 +619,7 @@ class Ingo_Storage_forward extends Ingo_Storage_rule {
  * Ingo_Storage_vacation is the object used to hold vacation rule
  * information.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Ingo 1.0
  * @package Ingo
  */
@@ -628,9 +629,11 @@ class Ingo_Storage_vacation extends Ingo_Storage_rule {
     var $_days = 7;
     var $_excludes = array();
     var $_ignorelist = true;
-    var $_obtype = INGO_STORAGE_ACTION_VACATION;
     var $_reason = '';
     var $_subject = '';
+    var $_start;
+    var $_end;
+    var $_obtype = INGO_STORAGE_ACTION_VACATION;
 
     function setVacationAddresses($data, $sort = true)
     {
@@ -662,9 +665,27 @@ class Ingo_Storage_vacation extends Ingo_Storage_rule {
         $this->_subject = $data;
     }
 
+    function setVacationStart($data)
+    {
+        $this->_start = $data;
+    }
+
+    function setVacationEnd($data)
+    {
+        $this->_end = $data;
+    }
+
     function getVacationAddresses()
     {
-        return $this->_addr;
+        if (empty($GLOBALS['conf']['hooks']['vacation_addresses'])) {
+            return $this->_addr;
+        }
+
+        $addresses = Horde::callHook('_ingo_hook_vacation_addresses', array(Ingo::getUser()), 'ingo');
+        if (is_a($addresses, 'PEAR_Error')) {
+            $addresses = array();
+        }
+        return $addresses;
     }
 
     function getVacationDays()
@@ -692,13 +713,110 @@ class Ingo_Storage_vacation extends Ingo_Storage_rule {
         return $this->_subject;
     }
 
+    function getVacationStart()
+    {
+        return $this->_start;
+    }
+
+    function getVacationStartYear()
+    {
+        return date('Y', $this->_start);
+    }
+
+    function getVacationStartMonth()
+    {
+        return date('n', $this->_start);
+    }
+
+    function getVacationStartDay()
+    {
+        return date('j', $this->_start);
+    }
+
+    function getVacationEnd()
+    {
+        return $this->_end;
+    }
+
+    function getVacationEndYear()
+    {
+        return date('Y', $this->_end);
+    }
+
+    function getVacationEndMonth()
+    {
+        return date('n', $this->_end);
+    }
+
+    function getVacationEndDay()
+    {
+        return date('j', $this->_end);
+    }
+
+}
+
+/**
+ * Ingo_Storage_spam is an object used to hold default spam-rule filtering
+ * information.
+ *
+ * @author  Jason M. Felice <jason.m.felice@gmail.com>
+ * @since   Ingo 1.2
+ * @package Ingo
+ */
+class Ingo_Storage_spam extends Ingo_Storage_rule {
+
+    /**
+     * The object type.
+     *
+     * @var integer
+     */
+    var $_obtype = INGO_STORAGE_ACTION_SPAM;
+
+    var $_folder = null;
+    var $_level = 5;
+
+    function Ingo_Storage_spam()
+    {
+        // Attempt to get the default Spam folder from imp
+        if (in_array('imp', $GLOBALS['registry']->listApps())) {
+            $prefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'],
+                                       'imp', Ingo::getUser(), '', null, false);
+            $prefs->retrieve();
+
+            $folder = $prefs->getValue('spam_folder', false);
+            if ($folder !== false) {
+                $this->_folder = $folder;
+            }
+        }
+    }
+
+    function setSpamFolder($folder)
+    {
+        $this->_folder = $folder;
+    }
+
+    function setSpamLevel($level)
+    {
+        $this->_level = $level;
+    }
+
+    function getSpamFolder()
+    {
+        return $this->_folder;
+    }
+
+    function getSpamLevel()
+    {
+        return $this->_level;
+    }
+
 }
 
 /**
  * Ingo_Storage_filters is the object used to hold user-defined filtering rule
  * information.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Ingo 1.0
  * @package Ingo
  */
