@@ -1,12 +1,14 @@
 <?php
 /**
- * $Horde: ingo/rule.php,v 1.63.8.8 2007/01/02 13:55:02 jan Exp $
+ * $Horde: ingo/rule.php,v 1.63.8.10 2008/01/02 11:32:08 jan Exp $
  *
- * Copyright 2002-2007 Mike Cochrane <mike@graftonhall.co.nz>
- * Copyright 2003-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/asl.php.
+ *
+ * @author Mike Cochrane <mike@graftonhall.co.nz>
+ * @author Michael Slusarz <slusarz@horde.org>
  */
 
 /* Define the key to use to indicate a user-defined header is requested. */
@@ -28,7 +30,7 @@ if (!Ingo::hasPermission('allow_rules')) {
 }
 
 /* Load the Ingo_Script:: driver. */
-$ingo_script = &Ingo::loadIngoScript();
+$ingo_script = Ingo::loadIngoScript();
 
 /* Redirect if no rules are available. */
 $availActions = $ingo_script->availableActions();
@@ -52,7 +54,13 @@ case 'create_folder':
 case 'rule_save':
 case 'rule_update':
 case 'rule_delete':
+    if (!Ingo::hasPermission('shares', PERMS_EDIT)) {
+        $notification->push(_("You do not have permission to edit filter rules."), 'horde.error');
+        header('Location: ' . Horde::applicationUrl('filters.php', true));
+        exit;
+    }
     $rule = array();
+    $rule['id'] = Util::getFormData('id');
     $rule['name'] = Util::getFormData('name');
     $rule['combine'] = Util::getFormData('combine');
     $rule['conditions'] = array();
@@ -64,6 +72,8 @@ case 'rule_delete':
     if ($ingo_script->caseSensitive()) {
         $casesensitive = Util::getFormData('case');
     }
+
+    $valid = true;
     foreach ($field as $key => $val) {
         if (!empty($val)) {
             $condition = array();
@@ -78,7 +88,16 @@ case 'rule_delete':
                 $condition['type'] = $ingo_fields[$val]['type'];
             }
             $condition['match'] = isset($match[$key]) ? $match[$key] : '';
+
+            if ($actionID == 'rule_save'
+                && empty($value[$key])
+                && $condition['match'] != 'exists'
+                && $condition['match'] != 'not exist') {
+                $notification->push(sprintf(_("You cannot create empty conditions. Please fill in a value for \"%s\"."), $condition['field']), 'horde.error');
+                $valid = false;
+            }
             $condition['value'] = isset($value[$key]) ? $value[$key] : '';
+
             if (isset($casesensitive)) {
                 $condition['case'] = isset($casesensitive[$key]) ? $casesensitive[$key] : '';
             }
@@ -107,7 +126,7 @@ case 'rule_delete':
     $_SESSION['ingo']['change'] = time();
 
     /* Save the rule. */
-    if ($actionID == 'rule_save') {
+    if ($actionID == 'rule_save' && $valid) {
         if (is_null($edit_number)) {
             if (Ingo::hasPermission('max_rules') !== true &&
                 Ingo::hasPermission('max_rules') <= count($filters->getFilterList())) {
@@ -128,16 +147,25 @@ case 'rule_delete':
         header('Location: ' . Horde::applicationUrl('filters.php'));
         exit;
     } elseif ($actionID == 'rule_delete') {
+        if (!Ingo::hasPermission('shares', PERMS_DELETE)) {
+            $notification->push(_("You do not have permission to delete filter rules."), 'horde.error');
+            header('Location: ' . Horde::applicationUrl('filters.php', true));
+            exit;
+        }
         $cond_num = Util::getFormData('conditionnumber');
         if (!is_null($cond_num)) {
             unset($rule['conditions'][$cond_num]);
             $rule['conditions'] = array_values($rule['conditions']);
         }
     }
-
     break;
 
 default:
+    if (!Ingo::hasPermission('shares', PERMS_EDIT)) {
+        $notification->push(_("You do not have permission to edit filter rules."), 'horde.error');
+        header('Location: ' . Horde::applicationUrl('filters.php', true));
+        exit;
+    }
     if (is_null($edit_number)) {
         if (Ingo::hasPermission('max_rules') !== true &&
             Ingo::hasPermission('max_rules') <= count($filters->getFilterList())) {
@@ -180,15 +208,15 @@ $lastcond = count($rule['conditions']) - 1;
 
 /* Display the conditions. */
 foreach ($rule['conditions'] as $cond_num => $condition) {
-    $lastfield = ($lastcond == $cond_num) ? true : false;
+    $lastfield = ($lastcond == $cond_num);
 
     /* Create the field listing. */
     $field_select = '';
     $option_selected = !isset($condition['field']);
 
     if ($lastfield) {
-        $field_select .= '<option value="">' . _("Select a field") . "</option>\n";
-        $field_select .= "<option value=\"\"> --------- </option>\n";
+        $field_select .= '<option value="">' . _("Select a field") . "</option>\n" .
+            "<option value=\"\"> --------- </option>\n";
     }
 
     foreach ($ingo_fields as $key => $val) {
@@ -223,8 +251,8 @@ foreach ($rule['conditions'] as $cond_num => $condition) {
     /* Add user defined header option. */
     $header_entry = false;
     if ($conf['rules']['userheader']) {
-        $field_select .= "<option value=\"\"> --------- </option>\n";
-        $field_select .= '<option value="' . INGO_USER_HEADER . '"' . ((!$option_selected) ? ' selected="selected"' : '') . '>' . _("Self-Defined Header") . (($lastfield) ? '' : ':') . "</option>\n";
+        $field_select .= "<option value=\"\"> --------- </option>\n" .
+            '<option value="' . INGO_USER_HEADER . '"' . ((!$option_selected) ? ' selected="selected"' : '') . '>' . _("Self-Defined Header") . (($lastfield) ? '' : ':') . "</option>\n";
         if (!$option_selected) {
             $header_entry = true;
             $userheader = Util::getFormData('userheader');
@@ -251,11 +279,20 @@ foreach ($rule['conditions'] as $cond_num => $condition) {
     }
 
     $match_select = '';
+    $selected_test = null;
+    if (!empty($condition['match'])) {
+        $selected_test = $condition['match'];
+    }
 
     if (empty($avail_tests)) {
         $match_select = "<option value=\"\"> --------- </option>\n";
     } else {
+        $first_test = null;
         foreach ($avail_tests as $test) {
+            if (is_null($selected_test)) {
+                $selected_test = $test;
+            }
+
             $testOb = $ingo_storage->getTestInfo($test);
             $match_select .= '<option value="' . htmlspecialchars($test) . '"';
             if (!empty($condition['match'])) {
@@ -271,14 +308,18 @@ foreach ($rule['conditions'] as $cond_num => $condition) {
     $testOb = $ingo_storage->getTestInfo(!empty($condition['match']) ? $condition['match'] : 'contains');
     $value = isset($condition['value']) ? htmlspecialchars($condition['value']): '';
 
-    $match_value = '<input name="value[' . (int)$cond_num . ']" size="40" value="' . $value . '" />';
+    $match_value = '';
+    if (!in_array($selected_test, array('exists', 'not exist'))) {
+        $match_value = '<label for="value_' . (int)$cond_num . '" class="hidden">Value</label>' .
+            '<input id="value_' . (int)$cond_num . '" name="value[' . (int)$cond_num . ']" size="40" value="' . $value . '" />';
+    }
 
     switch ($testOb->type) {
     case 'text':
         if ($ingo_script->caseSensitive()) {
-            $match_value .= '<input type="checkbox" name="case[' . (int)$cond_num . ']" value="1" ';
-            $match_value .= (isset($condition['case']) && $condition['case']) ? 'checked="checked" ' : '';
-            $match_value .= '/> ' . _("Case Sensitive");
+            $match_value .= '<input type="checkbox" id="case_' . (int)$cond_num . '" name="case[' . (int)$cond_num . ']" value="1" ' .
+                ((isset($condition['case']) && $condition['case']) ? 'checked="checked" ' : '') .
+                '/> ' . Horde::label('case_' . (int)$cond_num, _("Case Sensitive"));
         }
         break;
     }
@@ -287,8 +328,7 @@ foreach ($rule['conditions'] as $cond_num => $condition) {
 
 /* Get the action select output. */
 $actionselect = '';
-$actionlist = $ingo_script->availableActions();
-foreach ($actionlist as $val) {
+foreach ($availActions as $val) {
     $actionselect .= '<option value="' . htmlspecialchars($val) . '"';
     $action = $ingo_storage->getActionInfo($val);
     if ($val == $rule['action']) {
@@ -300,15 +340,18 @@ foreach ($actionlist as $val) {
 }
 
 /* Get the action value output. */
+$actionvaluelabel = '';
 $actionvalue = '';
 switch ($current_action->type) {
 case 'folder':
+    $actionvaluelabel = '<label for="actionvalue" class="hidden">' . _("Select target folder") . '</label>';
     $actionvalue = Ingo::flistSelect($rule['action-value'], 'rule');
     break;
 
 case 'text':
 case 'int':
-    $actionvalue = '<input name="actionvalue" size="40" value="' . htmlspecialchars($rule['action-value']) . '" />';
+    $actionvaluelabel = '<label for="actionvalue" class="hidden">' . _("Value") . '</label>';
+    $actionvalue = '<input id="actionvalue" name="actionvalue" size="40" value="' . htmlspecialchars($rule['action-value']) . '" />';
     break;
 }
 
