@@ -1,13 +1,14 @@
 <?php
 /**
- * $Horde: ingo/test.php,v 1.16.12.4 2007/01/02 13:55:02 jan Exp $
+ * $Horde: ingo/test.php,v 1.16.12.6 2008/01/02 11:32:08 jan Exp $
  *
- * Copyright 2002-2007 Brent J. Nordquist <bjn@horde.org>
- * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/asl.php.
+ *
+ * @author Brent J. Nordquist <bjn@horde.org>
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 /* Include Horde's core.php file. */
@@ -17,17 +18,17 @@ include_once '../lib/core.php';
  * package, in core.php. If String:: isn't defined, then we're not
  * finding some critical libraries. */
 if (!class_exists('String')) {
-    echo '<br /><span style="color: red; font-size: 18px; font-weight: bold;">The Horde_Util package was not found. If PHP\'s error_reporting setting is high enough, there should be error messages printed above that may help you in debugging the problem. If you are simply missing these files, then you need to get the <a href="http://cvs.horde.org/cvs.php/framework">framework</a> module from <a href="http://horde.org/source/">Horde CVS</a>, and install the packages in it with the install-packages.php script.</span>';
+    echo '<br /><h2 style="color:red">The Horde_Util package was not found. If PHP\'s error_reporting setting is high enough and display_errors is on, there should be error messages printed above that may help you in debugging the problem. If you are simply missing these files, then you need to get the <a href="http://cvs.horde.org/cvs.php/framework">framework</a> module from <a href="http://www.horde.org/source/">Horde CVS</a>, and install the packages in it with the install-packages.php script.</h2>';
     exit;
 }
 
 /* Initialize the Horde_Test:: class. */
-if (!(@is_readable('../lib/Test.php'))) {
+if (!is_readable('../lib/Test.php')) {
     echo 'ERROR: You must install Horde before running this script.';
     exit;
 }
 require_once '../lib/Test.php';
-$horde_test = &new Horde_Test;
+$horde_test = new Horde_Test;
 
 /* Ingo version. */
 $module = 'Ingo';
@@ -62,7 +63,11 @@ $module_list = array(
     'imap' => array(
         'descrip' => 'IMAP Support',
         'error' => 'If you will be using the IMAP client-side driver, PHP must have IMAP support built in. Compile PHP <code>--with-imap</code> before continuing.'
-    )
+    ),
+    'ssh2' => array(
+        'descrip' => 'SSH2 Support',
+        'error' => 'You need the SSH2 PECL module if you plan to use the SSH2 VFS driver to store procmail scripts on the mail server.'
+    ),
 );
 
 /* Display PHP Version information. */
